package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.generator.config.IFileCreate;
import com.google.gson.JsonObject;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.constant.ErrorCode;
import com.jwl.driver.server.dto.QuestionQueryDto;
import com.jwl.driver.server.dto.TestQuestionTypeDto;
import com.jwl.driver.server.entity.TdQuestion;
import com.jwl.driver.server.entity.TdSysConfig;
import com.jwl.driver.server.entity.TdSysConfigList;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdQuestionMapper;
import com.jwl.driver.server.redis.RedisCache;
import com.jwl.driver.server.service.ITdQuestionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdQuestionVersionService;
import com.jwl.driver.server.service.ITdSysConfigListService;
import com.jwl.driver.server.service.ITdSysConfigService;
import com.jwl.driver.server.vo.QuestionVo;
import com.jwl.driver.server.vo.QusetionCategoryVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 题库; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class TdQuestionServiceImpl extends ServiceImpl<TdQuestionMapper, TdQuestion> implements ITdQuestionService {

    @Autowired
    private ITdSysConfigService configService;

    @Autowired
    private ITdSysConfigListService configListService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ITdQuestionVersionService versionService;

    @Override
    public QuestionVo queryQuestionById(QuestionQueryDto queryDto) {

        QuestionQueryDto newDto = new QuestionQueryDto()
                .setQuestionId(queryDto.getQuestionId());
        List<QuestionVo> questionVos = queryQuestion(newDto);
        return CollectionUtil.isEmpty(questionVos) ? null : questionVos.get(0);
    }

    @Override
    public List<QuestionVo> queryQuestionByIdList(QuestionQueryDto queryDto) {

        QuestionQueryDto newDto = new QuestionQueryDto()
                .setQuestionIdList(queryDto.getQuestionIdList());
        return queryQuestion(newDto);
    }

    @Override
    public List<QuestionVo> queryQuestion(QuestionQueryDto queryDto) {
        //校验题库版本是否一致
        checkQuestionVersion(queryDto.getCarTypeId(), queryDto.getVersionId());

        List<QuestionVo> questionVos = this.getBaseMapper().queryQuestion(queryDto);
        questionOptionHandler(questionVos);
        return questionVos;
    }

    @Override
    public List<Long> queryQuestionId(QuestionQueryDto queryDto) {
        //校验题库版本是否一致
        checkQuestionVersion(queryDto.getCarTypeId(), queryDto.getVersionId());

        List<Long> questionIds = this.getBaseMapper().queryQuestionId(queryDto);
        return questionIds;
    }

    @Override
    public List<QuestionVo> getTestQuestion(QuestionQueryDto queryDto) {

        //校验题库版本是否一致
        checkQuestionVersion(queryDto.getCarTypeId(), queryDto.getVersionId());

        List<QuestionVo> resultList = new ArrayList<>();

        TdSysConfig tdSysConfig = configService.queryConfigByKey(Constants.TEST_QUESTION_TYPE_NUM, queryDto.getCarTypeId());
        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("获取信息异常");
        }

        List<TestQuestionTypeDto> testQuestionTypeDtos = JSON.parseArray(tdSysConfig.getConfigJson(), TestQuestionTypeDto.class);
        Optional<TestQuestionTypeDto> optional = testQuestionTypeDtos.stream().filter(s -> StrUtil.equals(s.getSubject(), queryDto.getSubject())).findFirst();
        if (optional.isPresent()) {
            TestQuestionTypeDto testQuestionTypeDto = optional.get();
            //判断题
            if (Objects.nonNull(testQuestionTypeDto.getJudgeNum()) && testQuestionTypeDto.getJudgeNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_ONE)
                        .setNum(testQuestionTypeDto.getJudgeNum());
                resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            }
            //单选
            if (Objects.nonNull(testQuestionTypeDto.getRadioNum()) && testQuestionTypeDto.getRadioNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_TWO)
                        .setNum(testQuestionTypeDto.getRadioNum());
                resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            }

            //多选
            if (Objects.nonNull(testQuestionTypeDto.getMultipleChoiceNum()) && testQuestionTypeDto.getMultipleChoiceNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_THREE)
                        .setNum(testQuestionTypeDto.getMultipleChoiceNum());
                resultList.addAll(this.getBaseMapper().queryQuestionByRandom(queryDto));
            }
        }
        questionOptionHandler(resultList);
        return resultList;
    }

    @Override
    public List<Long> getTestQuestionId(QuestionQueryDto queryDto) {
        //校验题库版本是否一致
        checkQuestionVersion(queryDto.getCarTypeId(), queryDto.getVersionId());

        List<Long> resultList = new ArrayList<>();

        TdSysConfig tdSysConfig = configService.queryConfigByKey(Constants.TEST_QUESTION_TYPE_NUM, queryDto.getCarTypeId());
        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("获取信息异常");
        }

        List<TestQuestionTypeDto> testQuestionTypeDtos = JSON.parseArray(tdSysConfig.getConfigJson(), TestQuestionTypeDto.class);
        Optional<TestQuestionTypeDto> optional = testQuestionTypeDtos.stream().filter(s -> StrUtil.equals(s.getSubject(), queryDto.getSubject())).findFirst();
        if (optional.isPresent()) {
            TestQuestionTypeDto testQuestionTypeDto = optional.get();
            //判断题
            if (Objects.nonNull(testQuestionTypeDto.getJudgeNum()) && testQuestionTypeDto.getJudgeNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_ONE)
                        .setNum(testQuestionTypeDto.getJudgeNum());
                resultList.addAll(this.getBaseMapper().queryQuestionIdByRandom(queryDto));
            }
            //单选
            if (Objects.nonNull(testQuestionTypeDto.getRadioNum()) && testQuestionTypeDto.getRadioNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_TWO)
                        .setNum(testQuestionTypeDto.getRadioNum());
                resultList.addAll(this.getBaseMapper().queryQuestionIdByRandom(queryDto));
            }

            //多选
            if (Objects.nonNull(testQuestionTypeDto.getMultipleChoiceNum()) && testQuestionTypeDto.getMultipleChoiceNum() > 0) {
                queryDto.setType(Constants.QUESTION_TYPE_THREE)
                        .setNum(testQuestionTypeDto.getMultipleChoiceNum());
                resultList.addAll(this.getBaseMapper().queryQuestionIdByRandom(queryDto));
            }
        }
        return resultList;
    }

    @Override
    public List<QusetionCategoryVo> questionCategory(QuestionQueryDto queryDto) {

        List<QusetionCategoryVo> resultList = new ArrayList<>();
        //获取题目
        List<QuestionVo> tdQuestionVos = queryQuestionByIdList(queryDto);
        if (CollectionUtil.isEmpty(tdQuestionVos)) {
            return resultList;
        }
        //获取分类
        List<TdSysConfigList> categoryList = configListService.querySysConfigList(Constants.QUESTION_CATEGORY, Constants.DEFAULT_CARTYPE_ID);
        Map<String, String> categoryMap = categoryList.stream().collect(Collectors.toMap(TdSysConfigList::getConfigItemCode, TdSysConfigList::getConfigItemName, (v1, v2) -> v1));
        Map<String, Integer> totalMap = new HashMap<>();
        Map<String, List<Long>> errorQuestionIdMap = new HashMap<>();

        for (QuestionVo tdQuestionVo : tdQuestionVos) {
            String category = tdQuestionVo.getCategory();
            if (StrUtil.isBlank(category)) {
                continue;
            }
            String[] split = category.split(",");
            for (String s : split) { //遍历字符串数组
                if (totalMap.containsKey(s)) { //判断Map集合中是否有该字符
                    Integer value = totalMap.get(s); //通过key找出集合中的value
                    totalMap.put(s, value + 1); //将值的数据加1，然后添加到集合中去
                    List<Long> errorQuestionIdList = errorQuestionIdMap.get(s);
                    errorQuestionIdList.add(tdQuestionVo.getQuestionId());
                } else {
                    totalMap.put(s, 1); //此时集合中没有该Key,所以将该字符作为键加入到集合中
                    List<Long> errorQuestionIdList = new ArrayList<>();
                    errorQuestionIdList.add(tdQuestionVo.getQuestionId());
                    errorQuestionIdMap.put(s, errorQuestionIdList);
                }
            }
        }

        for (String category : totalMap.keySet()) {
            QusetionCategoryVo categoryVo = new QusetionCategoryVo()
                    .setCategory(category)
                    .setErrorQuestionIdList(errorQuestionIdMap.get(category))
                    .setCategoryName(categoryMap.getOrDefault(category, "其他类型"))
                    .setNum(totalMap.get(category));
            resultList.add(categoryVo);
        }
        return resultList;
    }

    @Override
    public int updateQuestion(QuestionVo questionVo) {
        return this.getBaseMapper().updateQuestion(questionVo);
    }

    @Override
    public Map<String, Integer> querySpecialNum(QuestionQueryDto queryDto) {
        //校验题库版本是否一致
        checkQuestionVersion(queryDto.getCarTypeId(), queryDto.getVersionId());

        Map<String, Integer> resultMap = redisCache.getCacheObject(Constants.SPECIAL_NUM_PREFIX + queryDto.getCarTypeId() + "_" + queryDto.getSubject());
        if (Objects.nonNull(resultMap)) {
            return resultMap;
        }
        resultMap = this.getBaseMapper().querySpecialNum(queryDto);

        if (CollectionUtil.isEmpty(resultMap)) {
            return resultMap;
        }

        TdSysConfig tdSysConfig = configService.queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_CARTYPE_ID);
        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("系统异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.SPECIAL_NUM_PREFIX + queryDto.getCarTypeId() + "_" + queryDto.getSubject(), resultMap, Long.parseLong(tdSysConfig.getConfigValue()));
        return resultMap;
    }

    /**
     * 问题选项规整
     */
    private void questionOptionHandler(List<QuestionVo> questionVos) {
        if (CollectionUtil.isNotEmpty(questionVos)) {
            for (QuestionVo questionVo : questionVos) {
                //如果选项A没有值 则为判断题
                List<QuestionVo.QuestionOption> optionList = new ArrayList<>();
                if (StrUtil.isBlank(questionVo.getChooseA()) || StrUtil.equals(questionVo.getType(), Constants.QUESTION_TYPE_ONE)) {
                    optionList.add(new QuestionVo.QuestionOption("A", "正确", "1"));
                    optionList.add(new QuestionVo.QuestionOption("B", "错误", "2"));
                } else {
                    //起码ABCD都有
                    optionList.add(new QuestionVo.QuestionOption("A", questionVo.getChooseA(), "1"));
                    optionList.add(new QuestionVo.QuestionOption("B", questionVo.getChooseB(), "2"));
                    optionList.add(new QuestionVo.QuestionOption("C", questionVo.getChooseC(), "3"));
                    optionList.add(new QuestionVo.QuestionOption("D", questionVo.getChooseD(), "4"));

                    if (StrUtil.isNotBlank(questionVo.getChooseE())) {
                        optionList.add(new QuestionVo.QuestionOption("E", questionVo.getChooseE(), "5"));
                    }
                    if (StrUtil.isNotBlank(questionVo.getChooseF())) {
                        optionList.add(new QuestionVo.QuestionOption("F", questionVo.getChooseF(), "6"));
                    }
                    if (StrUtil.isNotBlank(questionVo.getChooseG())) {
                        optionList.add(new QuestionVo.QuestionOption("G", questionVo.getChooseG(), "7"));
                    }
                }
                questionVo.setOptionList(optionList);
            }
        }
    }

    /**
     * 校验题库是否一直
     *
     * @param carTypeId
     * @param version
     */
    private void checkQuestionVersion(Integer carTypeId, Long version) {
        if (Objects.isNull(carTypeId) || Objects.isNull(version)) {
            return;
        }

        Long versionId = versionService.currentVersion(carTypeId);
        if (!Objects.equals(versionId, version)) {
            throw new BusinessException(ErrorCode.VERSION_ERROR, "题库版本不一致");
        }
    }

    public static void main(String[] args) {
        List<TestQuestionTypeDto> testQuestionTypeDtos = new ArrayList<>();
        TestQuestionTypeDto dto1 = new TestQuestionTypeDto()
                .setSubject("1")
                .setJudgeNum(40)
                .setRadioNum(60)
                .setMultipleChoiceNum(0);
        TestQuestionTypeDto dto2 = new TestQuestionTypeDto()
                .setSubject("4")
                .setJudgeNum(20)
                .setRadioNum(20)
                .setMultipleChoiceNum(10);
        testQuestionTypeDtos.add(dto1);
        testQuestionTypeDtos.add(dto2);
        String jsonString = JSONArray.toJSONString(testQuestionTypeDtos);
        System.out.println(jsonString);

        List<TestQuestionTypeDto> testQuestionTypeDtos1 = JSONArray.parseArray(jsonString, TestQuestionTypeDto.class);
        System.out.println(testQuestionTypeDtos1);
    }
}
