package com.jwl.driver.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.entity.TdQuestionVersion;
import com.jwl.driver.server.entity.TdSysConfig;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdQuestionVersionMapper;
import com.jwl.driver.server.redis.RedisCache;
import com.jwl.driver.server.service.ITdQuestionVersionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdSysConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * <p>
 * 题库版本表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-09-03
 */
@Service
@Slf4j
public class TdQuestionVersionServiceImpl extends ServiceImpl<TdQuestionVersionMapper, TdQuestionVersion> implements ITdQuestionVersionService {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ITdSysConfigService configService;

    @Override
    public Long currentVersion(Integer carTypeId) {

        Integer value = redisCache.getCacheObject(Constants.QUESTION_VERSOPN_PREFIX + carTypeId);

        if (value !=null){
            return value.longValue();
        }

        LambdaQueryWrapper<TdQuestionVersion> cond = new LambdaQueryWrapper<TdQuestionVersion>()
                .eq(TdQuestionVersion::getCarTypeId,carTypeId)
                .eq(TdQuestionVersion::getIsActive,Constants.IS_ACTIVE_TRUE);

        TdQuestionVersion tdQuestionVersion = this.getBaseMapper().selectOne(cond);
        if (Objects.isNull(tdQuestionVersion)){
            throw new BusinessException("题库版本异常");
        }

        TdSysConfig tdSysConfig = configService.queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_CARTYPE_ID);
        if (Objects.isNull(tdSysConfig)){
            log.info("config is not exist");
            throw new BusinessException("系统未知异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.QUESTION_VERSOPN_PREFIX + carTypeId,tdQuestionVersion.getVersionId(),Long.parseLong(tdSysConfig.getConfigValue()));
        return tdQuestionVersion.getVersionId();
    }

    @Override
    public Boolean addVersion(TdQuestionVersion versionDto) {

        LambdaQueryWrapper<TdQuestionVersion> cond = new LambdaQueryWrapper<TdQuestionVersion>()
                .eq(TdQuestionVersion::getCarTypeId,versionDto.getCarTypeId())
                .eq(TdQuestionVersion::getIsActive,Constants.IS_ACTIVE_TRUE);

        TdQuestionVersion tdQuestionVersion = this.getBaseMapper().selectOne(cond);

        if (Objects.nonNull(tdQuestionVersion)){
            tdQuestionVersion.setIsActive(Constants.IS_ACTIVE_FALSE);
            this.getBaseMapper().updateById(tdQuestionVersion);
        }
        TdQuestionVersion newVersion = new TdQuestionVersion();
        BeanUtils.copyProperties(versionDto,newVersion);
        newVersion.setCreateTime(LocalDateTime.now())
                .setIsActive(Constants.IS_ACTIVE_TRUE);
        this.baseMapper.insert(newVersion);
        //删除原redis
        redisCache.deleteObject(Constants.QUESTION_VERSOPN_PREFIX + versionDto.getCarTypeId());
        redisCache.deleteObjectLike(Constants.SPECIAL_NUM_PREFIX + versionDto.getCarTypeId() );
        return Boolean.TRUE;
    }
}
