package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.entity.TdSysConfig;
import com.jwl.driver.server.entity.TdSysConfigList;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdSysConfigListMapper;
import com.jwl.driver.server.redis.RedisCache;
import com.jwl.driver.server.service.ITdSysConfigListService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdSysConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 系统配置列表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class TdSysConfigListServiceImpl extends ServiceImpl<TdSysConfigListMapper, TdSysConfigList> implements ITdSysConfigListService {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ITdSysConfigService configService;

    @Override
    public List<TdSysConfigList> querySysConfigList(String configKey, Integer carTypeId) {

        List<TdSysConfigList> configLists = redisCache.getCacheObject(Constants.CONFIG_LIST_PREFIX + configKey + "_" + carTypeId);
        if (CollectionUtil.isNotEmpty(configLists)) {
            return configLists;
        }
        //redis中没有则查询数据库
        LambdaQueryWrapper<TdSysConfigList> queryWrapper = new LambdaQueryWrapper<TdSysConfigList>()
                .eq(TdSysConfigList::getCarTypeId, carTypeId)
                .eq(TdSysConfigList::getConfigKey, configKey)
                .eq(TdSysConfigList::getIsActive, Constants.IS_ACTIVE_TRUE)
                .orderByAsc(TdSysConfigList::getShowOrder);
        configLists = this.getBaseMapper().selectList(queryWrapper);

        if (CollectionUtil.isEmpty(configLists)) {
            return configLists;
        }

        //获取redis缓存时间
        TdSysConfig tdSysConfig = configService.queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_CARTYPE_ID);
        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("系统异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.CONFIG_LIST_PREFIX + configKey + "_" + carTypeId, configLists, Long.parseLong(tdSysConfig.getConfigValue()));

        return configLists;
    }

    @Override
    public Map<String, Map<String, String>> querySysConfigMap(List<String> configKeyList, Integer carTypeId) {

        Map<String, Map<String, String>> resultMap = new HashMap<>();
        LambdaQueryWrapper<TdSysConfigList> queryWrapper = new LambdaQueryWrapper<TdSysConfigList>()
                .eq(TdSysConfigList::getCarTypeId, carTypeId)
                .in(TdSysConfigList::getConfigKey, configKeyList)
                .eq(TdSysConfigList::getIsActive, Constants.IS_ACTIVE_TRUE);

        List<TdSysConfigList> configLists = this.getBaseMapper().selectList(queryWrapper);
        for (String configKey : configKeyList) {
            Map<String, String> collect = configLists.stream().filter(s -> StrUtil.equals(s.getConfigKey(), configKey))
                    .sorted(Comparator.comparing(TdSysConfigList::getShowOrder))
                    .collect(Collectors.toMap(TdSysConfigList::getConfigItemCode, TdSysConfigList::getConfigItemName));
            resultMap.put(configKey, collect);
        }
        return resultMap;
    }
}
