package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.entity.TdSysConfig;
import com.jwl.driver.server.entity.TdSysConfigList;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdSysConfigMapper;
import com.jwl.driver.server.redis.RedisCache;
import com.jwl.driver.server.service.ITdSysConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 系统配置表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class TdSysConfigServiceImpl extends ServiceImpl<TdSysConfigMapper, TdSysConfig> implements ITdSysConfigService {

    @Autowired
    private RedisCache redisCache;

    @Override
    public TdSysConfig queryConfigByKey(String configKey, Integer carTypeId) {
        log.info("获取系统配置参数，configKey：{}", configKey);

        TdSysConfig config = redisCache.getCacheObject(Constants.CONFIG_PREFIX + configKey + "_" + carTypeId);
        if (Objects.nonNull(config)){
            return config;
        }

        LambdaQueryWrapper<TdSysConfig> queryWrapper = new QueryWrapper<TdSysConfig>().lambda()
                .eq(TdSysConfig::getCarTypeId,carTypeId)
                .eq(TdSysConfig::getConfigKey, configKey)
                .eq(TdSysConfig::getIsActive, Constants.IS_ACTIVE_TRUE);
        config = getOne(queryWrapper);

        if (Objects.isNull(config)){
            return config;
        }

        //获取redis缓存时间
        TdSysConfig tdSysConfig = null;
        if (StrUtil.equals(configKey,Constants.REDIS_EXPIRE_TIME)){
            tdSysConfig = config;
        }else {
            tdSysConfig = queryConfigByKey(Constants.REDIS_EXPIRE_TIME, Constants.DEFAULT_CARTYPE_ID);
        }

        if (Objects.isNull(tdSysConfig)) {
            log.info("config is not exist");
            throw new BusinessException("系统异常");
        }
        // 塞入redis中
        redisCache.setCacheObject(Constants.CONFIG_PREFIX + configKey + "_" + carTypeId, config, Long.parseLong(tdSysConfig.getConfigValue()));

        return config;
    }

    @Override
    public String queryConfigValueByKey(String configKey, Integer carTypeId) {
        log.info("获取系统配置参数，configKey：{}", configKey);
        TdSysConfig config = queryConfigByKey(configKey,carTypeId);
        if(null == config) {
            return null;
        }
        return config.getConfigValue();
    }

    @Override
    public Map<String, TdSysConfig> queryConfigByKeys(List<String> configKeyList,Integer carTypeId) {
        log.info("获取系统配置参数，configKeyList：{}", configKeyList);
        LambdaQueryWrapper<TdSysConfig> queryWrapper = new QueryWrapper<TdSysConfig>().lambda()
                .eq(TdSysConfig::getCarTypeId,carTypeId)
                .in(TdSysConfig::getConfigKey, configKeyList)
                .eq(TdSysConfig::getIsActive, Constants.IS_ACTIVE_TRUE);
        List<TdSysConfig> configList = list(queryWrapper);

        if(CollectionUtil.isEmpty(configList)) {
            return Collections.EMPTY_MAP;
        }
        return configList.stream().collect(Collectors.toMap(TdSysConfig::getConfigKey, sysConfig -> sysConfig));
    }
}
